package org.jtheque.movies;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.ISchemaManager;
import org.jtheque.core.managers.feature.Feature;
import org.jtheque.core.managers.feature.IFeatureManager;
import org.jtheque.core.managers.feature.Feature.FeatureType;
import org.jtheque.core.managers.module.beans.CollectionBasedModule;
import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.PrimaryModule;
import org.jtheque.core.managers.module.annotations.Plug;
import org.jtheque.core.managers.module.annotations.PrePlug;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.module.annotations.UnPlug;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.able.components.TabComponent;
import org.jtheque.movies.persistence.MoviesSchema;
import org.jtheque.movies.services.able.ICategoriesService;
import org.jtheque.movies.services.able.IMoviesService;
import org.jtheque.primary.PrimaryUtils;
import org.jtheque.primary.services.able.ICollectionsService;
import org.jtheque.primary.utils.DataTypeManager;
import org.jtheque.primary.view.impl.choice.ChoiceAction;
import org.jtheque.primary.view.impl.choice.ChoiceActionFactory;
import org.jtheque.primary.view.impl.sort.Sorter;
import org.jtheque.primary.view.impl.sort.SorterFactory;

/**
 * A JTheque Module for managing movies.
 *
 * @author Baptiste Wicht
 */
@Module(name = "JTheque Movies Module", description = "Module de gestion de vidos", author = "Baptiste Wicht", 
    version = "1.2", coreVersion = "2.0.1", jarFile = "jtheque-movies-module-1.2.jar")
@PrimaryModule(icon = "org/jtheque/movies/ressources/images/MoviesModule.png")
@ModuleDefinition(updateURL = "http://jtheque.developpez.com/public/versions/MoviesModule.versions")
public final class MoviesModule implements CollectionBasedModule {
    public static final String IMAGES_BASENAME = "org/jtheque/movies/ressources/images";
    private static final String BASENAME = "org.jtheque.movies.ressources.i18n.movies";

    private Sorter[] sorters;
    private ChoiceAction[] choiceActions;

    private Feature categoriesFeature;

    private Schema schema;

    @PrePlug
    public void prePlug() {
        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Preplug Movies ModuleContainer Start");
        
        schema = new MoviesSchema();

        Managers.getManager(ISchemaManager.class).registerSchema(schema);
        
        Managers.getManager(ILanguageManager.class).addBasename(BASENAME);
        
        PrimaryUtils.setPrimaryImpl("Movies");
        PrimaryUtils.prePlug();

        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Preplug Movies ModuleContainer End");
    }

    @Plug
    public void plug() {
        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Plug Movies ModuleContainer Start");

        PrimaryUtils.plug();

        DataTypeManager.bindDataTypeToKey(ICollectionsService.DATA_TYPE, "data.titles.collection");
        DataTypeManager.bindDataTypeToKey(ICategoriesService.DATA_TYPE, "data.titles.category");
        DataTypeManager.bindDataTypeToKey(IMoviesService.DATA_TYPE, "data.titles.movie");
        
        for (Sorter sorter : sorters) {
            SorterFactory.getInstance().addSorter(sorter);
        }
        
        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Plug Movies ModuleContainer End");
    }

    @Override
    public boolean chooseCollection(String collection, String password, boolean create) {
        ICollectionsService collectionsService = (ICollectionsService)Managers.getManager(IBeansManager.class).getBean("collectionsService");

        if(create){
            collectionsService.createCollectionAndUse(collection, password);
        } else {
            if(!collectionsService.login(collection, password)){
                return false;
            }
        }
        
        return true;
    }

    @Override
    public void plugCollection() {
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.addChoiceAction(action);
        }

        Managers.getManager(IViewManager.class).addTabComponent((TabComponent) Managers.getManager(IBeansManager.class).getBean("movieView"));

        IFeatureManager manager = Managers.getManager(IFeatureManager.class);
        
        categoriesFeature = manager.createFeature(500, FeatureType.PACK, "category.menu.title");

        manager.addSubFeature(categoriesFeature, "newCategoryAction", FeatureType.ACTION, 1);
        manager.addSubFeature(categoriesFeature, "editCategoryAction", FeatureType.ACTION, 2);
        manager.addSubFeature(categoriesFeature, "deleteCategoryAction", FeatureType.ACTION, 3);
        
        Managers.getManager(IFeatureManager.class).addFeature(categoriesFeature);
    }

    @UnPlug
    public void unplug() {
        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Unplug Movies ModuleContainer Start");

        Managers.getManager(IFeatureManager.class).removeFeature(categoriesFeature);
        
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.removeChoiceAction(action);
        }

        DataTypeManager.unbindDataType(ICategoriesService.DATA_TYPE);
        DataTypeManager.unbindDataType(IMoviesService.DATA_TYPE);
        DataTypeManager.unbindDataType(ICollectionsService.DATA_TYPE);

        for (Sorter sorter : sorters) {
            SorterFactory.getInstance().removeSorter(sorter);
        }

        Managers.getManager(IViewManager.class).removeTabComponent((TabComponent) Managers.getManager(IBeansManager.class).getBean("movieView"));

        PrimaryUtils.unplug();
        
        Managers.getManager(ILanguageManager.class).removeBasename(BASENAME);

        //Schema

        Managers.getManager(ISchemaManager.class).unregisterSchema(schema);

        Managers.getManager(ILoggingManager.class).getLogger(getClass()).trace("Unplug Movies ModuleContainer End");
    }

    public void setChoiceActions(ChoiceAction[] choiceActions) {
        this.choiceActions = choiceActions;
    }

    public void setSorters(Sorter[] sorters) {
        this.sorters = sorters;
    }
}
