package org.jtheque.movies.persistence;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.Insert;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.bean.Version;
import org.jtheque.movies.persistence.dao.able.IDaoMovies;
import org.jtheque.movies.persistence.dao.able.IDaoCategories;
import org.jtheque.primary.dao.able.IDaoCollections;
import org.jtheque.primary.PrimaryUtils;

import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;

/**
 * The database schema for the Movies Module.
 *
 * @author Baptiste Wicht
 */
public final class MoviesSchema implements Schema {
    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new MoviesSchema.
     */
    public MoviesSchema(){
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public Version getVersion() {
        return new Version("1.1");
    }

    @Override
    public String getName() {
        return "Movies-Schema";
    }

    @Override
    public void install() {
        createDataTables();
        createReferentialIntegrityConstraints();
    }

    @Override
    public void update(Version from) {
        if("1.0".equals(from.getVersion())){
            createReferentialIntegrityConstraints();            
        }
    }

    private void createDataTables() {
        jdbcTemplate.update("CREATE TABLE " + IDaoMovies.MOVIES_CATEGORIES_TABLE + " (THE_MOVIE_FK INT NOT NULL, THE_CATEGORY_FK INT NOT NULL)");
        jdbcTemplate.update("CREATE TABLE " + IDaoCategories.TABLE + " (ID INT IDENTITY PRIMARY KEY, TITLE VARCHAR(100) NOT NULL UNIQUE, THE_COLLECTION_FK INT NOT NULL)");
        jdbcTemplate.update("CREATE TABLE " + IDaoMovies.TABLE + " (ID INT IDENTITY PRIMARY KEY, TITLE VARCHAR(100) NOT NULL UNIQUE, NOTE INT NULL, FILE VARCHAR(200) NOT NULL, THE_COLLECTION_FK INT NOT NULL)");

        jdbcTemplate.update("CREATE INDEX MOVIES_IDX ON " + IDaoMovies.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX MOVIE_CATEGORIES_IDX ON " + IDaoCategories.TABLE + "(ID)");
    }

    private void createReferentialIntegrityConstraints() {
        jdbcTemplate.update("ALTER TABLE " + IDaoMovies.MOVIES_CATEGORIES_TABLE + " ADD FOREIGN KEY (THE_MOVIE_FK) REFERENCES  " + IDaoMovies.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoMovies.MOVIES_CATEGORIES_TABLE + " ADD FOREIGN KEY (THE_CATEGORY_FK) REFERENCES  " + IDaoCategories.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoCategories.TABLE + " ADD FOREIGN KEY (THE_COLLECTION_FK) REFERENCES  " + IDaoCollections.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoMovies.TABLE + " ADD FOREIGN KEY (THE_COLLECTION_FK) REFERENCES  " + IDaoCollections.TABLE + "  (ID) ON UPDATE SET NULL");
    }

    @Override
    public void importDataFromHSQL(Iterable<Insert> inserts) {
        for(Insert insert : inserts){
            if("MOVIE_CATEGORY".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoMovies.MOVIES_CATEGORIES_TABLE + " (THE_MOVIE_FK, THE_CATEGORY_FK) VALUES(?, ?)", insert.getInt(0), insert.getInt(1));
            } else if("OD_CATEGORY".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoCategories.TABLE + " (ID, TITLE, THE_COLLECTION_FK) VALUES(?, ?, ?)", insert.getInt(0), insert.getString(2), insert.getInt(3));
            } else if("OD_MOVIE_COLLECTION".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoCollections.TABLE + " (ID, TITLE, PROTECTED, PASSWORD, IMPL) VALUES(?,?,?,?,?)", 
                        insert.getInt(0), insert.getString(4), insert.getBoolean(3), insert.getString(2), PrimaryUtils.getPrimaryImpl());
            } else if("OD_MOVIE".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoMovies.TABLE + " (ID, TITLE, NOTE, FILE, THE_COLLECTION_FK) VALUES(?, ?, ?, ?, ?)",
                        insert.getInt(0), insert.getString(4), insert.getInt(3), insert.getString(2), insert.getInt(5));
            }
        }
    }
}