package org.jtheque.movies.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.movies.persistence.dao.able.IDaoCategories;
import org.jtheque.movies.persistence.dao.able.IDaoMovies;
import org.jtheque.movies.persistence.od.MovieCategoryRelation;
import org.jtheque.movies.persistence.od.MovieImpl;
import org.jtheque.movies.persistence.od.abstraction.Category;
import org.jtheque.movies.persistence.od.abstraction.Movie;
import org.jtheque.primary.od.abstraction.Data;
import org.jtheque.primary.dao.able.IDaoCollections;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Cette classe permet de grer la persistance des films.
 *
 * @author Baptiste Wicht
 */
public final class DaoMovies extends GenericDao<MovieImpl> implements IDaoMovies {
    private final ParameterizedRowMapper<MovieImpl> rowMapper = new MovieRowMapper();
    private final ParameterizedRowMapper<MovieCategoryRelation> relationRowMapper = new RelationRowMapper();
    private final QueryMapper queryMapper = new MovieQueryMapper();

    private List<MovieCategoryRelation> relationsToCategories;

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    @Resource
    private IDaoCollections daoCollections;

    @Resource
    private IDaoCategories daoCategories;

    /**
     * Construct a new DaoMovies.
     */
    public DaoMovies() {
        super(TABLE);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<MovieImpl> getMovies() {
        List<MovieImpl> films = (List<MovieImpl>) getMovies(daoCollections.getCurrentCollection());

        Collections.sort(films);

        return films;
    }

    @Override
    public List<MovieImpl> getAllMovies() {
        return getAll();
    }

    /**
     * Return all the films of the collection.
     *
     * @param collection The collection.
     * @return A List containing all the films of the collections.
     */
    @SuppressWarnings("unchecked")
    private List<? extends Data> getMovies(Entity collection) {
        if (collection == null || !collection.isSaved()) {
            return getAllMovies();
        }

        load();

        List<MovieImpl> movies = new ArrayList<MovieImpl>(getCache().size() / 2);

        for (MovieImpl movie : getCache().values()) {
            if (movie.getTheCollection().getId() == collection.getId()) {
                movies.add(movie);
            }
        }

        return movies;
    }

    @Override
    public boolean delete(MovieImpl movie) {
        boolean deleted = super.delete(movie);

        jdbcTemplate.update("DELETE FROM " + MOVIES_CATEGORIES_TABLE + " WHERE THE_MOVIE_FK = ?", movie.getId());

        return deleted;
    }

    @Override
    public void save(MovieImpl movie) {
        super.save(movie);

        jdbcTemplate.update("DELETE FROM " + MOVIES_CATEGORIES_TABLE + " WHERE THE_MOVIE_FK = ?", movie.getId());

        createLinks(movie);
    }

    /**
     * Create the links between the movie and the categories.
     *
     * @param movie The movie. 
     */
    private void createLinks(Movie movie) {
        for (Category category : movie.getCategories()) {
            jdbcTemplate.update("INSERT INTO " + MOVIES_CATEGORIES_TABLE + " (THE_MOVIE_FK, THE_CATEGORY_FK) VALUES(?,?)", movie.getId(), category.getId());
        }
    }

    @Override
    public void create(MovieImpl movie) {
        movie.setTheCollection(daoCollections.getCurrentCollection());

        //First we create the movie in the table
        super.create(movie);

        //next, we can create the links to the categories
        createLinks(movie);
    }

    @Override
    public void saveAll() {
        for (MovieImpl movie : getMovies()) {
            save(movie);
        }
    }

    @Override
    protected ParameterizedRowMapper<MovieImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        relationsToCategories = jdbcTemplate.query("SELECT * FROM " + MOVIES_CATEGORIES_TABLE, relationRowMapper);

        List<MovieImpl> movies = persistenceContext.getSortedList(TABLE, rowMapper);

        for (MovieImpl movie : movies) {
            getCache().put(movie.getId(), movie);
        }

        setCacheEntirelyLoaded(true);

        relationsToCategories.clear();
    }

    @Override
    protected void load(int i) {
        MovieImpl book = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, book);
    }

    /**
     * A row mapper to map a resultset to a reation between category and movie.
     *
     * @author Baptiste Wicht
     */
    private static final class RelationRowMapper implements ParameterizedRowMapper<MovieCategoryRelation> {
        @Override
        public MovieCategoryRelation mapRow(ResultSet rs, int i) throws SQLException {
            MovieCategoryRelation relation = new MovieCategoryRelation();

            relation.setMovie(rs.getInt("THE_MOVIE_FK"));
            relation.setCategory(rs.getInt("THE_CATEGORY_FK"));

            return relation;
        }
    }

    /**
     * A row mapper to map a resultset to a movie.
     *
     * @author Baptiste Wicht
     */
    private final class MovieRowMapper implements ParameterizedRowMapper<MovieImpl> {
        @Override
        public MovieImpl mapRow(ResultSet rs, int i) throws SQLException {
            MovieImpl movie = new MovieImpl();

            movie.setId(rs.getInt("ID"));
            movie.setTitle(rs.getString("TITLE"));
            movie.setNote(DaoNotes.getINSTANCE().getNote(NoteType.getEnum(rs.getInt("NOTE"))));
            movie.setFile(rs.getString("FILE"));
            movie.setTheCollection(daoCollections.getCollection(rs.getInt("THE_COLLECTION_FK")));

            if (relationsToCategories != null && !relationsToCategories.isEmpty()) {
                for (MovieCategoryRelation relation : relationsToCategories) {
                    if (relation.getMovie() == movie.getId()) {
                        movie.getCategories().add(daoCategories.getCategory(relation.getCategory()));
                    }
                }
            } else {
                relationsToCategories = jdbcTemplate.query("SELECT * FROM " + MOVIES_CATEGORIES_TABLE + " WHERE THE_MOVIE_FK = ?", relationRowMapper, movie.getId());

                for (MovieCategoryRelation relation : relationsToCategories) {
                    movie.getCategories().add(daoCategories.getCategory(relation.getCategory()));
                }

                relationsToCategories.clear();
            }

            return movie;
        }
    }

    /**
     * A query mapper to map movie to requests.
     *
     * @author Baptiste Wicht
     */
    private static final class MovieQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Movie movie = (Movie) entity;

            String query = "INSERT INTO " + TABLE + " (TITLE, NOTE, FILE, THE_COLLECTION_FK) VALUES(?,?,?,?)";

            Object[] parameters = {
                    movie.getTitle(),
                    movie.getNote().getValue().intValue(),
                    movie.getFile(),
                    movie.getTheCollection().getId()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Movie movie = (Movie) entity;

            String query = "UPDATE " + TABLE + " SET TITLE = ?, NOTE = ?, FILE = ?, THE_COLLECTION_FK = ? WHERE ID = ?";

            Object[] parameters = {
                    movie.getTitle(),
                    movie.getNote().getValue().intValue(),
                    movie.getFile(),
                    movie.getTheCollection().getId(),
                    movie.getId()
            };

            return new Query(query, parameters);
        }
    }
}