package org.jtheque.movies.views.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.swingx.JXTitledPanel;
import org.jdesktop.swingx.JXTree;
import org.jdesktop.swingx.border.DropShadowBorder;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.utils.ui.ValidationUtils;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.core.managers.view.impl.components.panel.FileChooserPanel;
import org.jtheque.movies.persistence.od.MovieImpl;
import org.jtheque.movies.services.able.IMoviesService;
import org.jtheque.movies.views.able.ICategoriesView;
import org.jtheque.movies.views.able.IMovieView;
import org.jtheque.movies.views.impl.fb.MovieFormBean;
import org.jtheque.movies.views.impl.models.MoviesModel;
import org.jtheque.movies.views.impl.toolbars.JPanelMovieToolBar;
import org.jtheque.movies.controllers.able.IMovieController;
import org.jtheque.primary.view.able.ToolbarView;
import org.jtheque.primary.view.impl.components.panels.JThequeTitledPanel;
import org.jtheque.primary.view.impl.components.panels.PrincipalDataPanel;
import org.jtheque.utils.ui.GridBagUtils;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.primary.view.impl.listeners.CurrentObjectListener;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.listeners.ViewStateListener;
import org.jtheque.primary.view.impl.models.NotesComboBoxModel;
import org.jtheque.primary.view.impl.renderers.JThequeTreeCellRenderer;
import org.jtheque.primary.view.impl.renderers.NoteComboRenderer;
import org.jtheque.primary.view.impl.sort.SortManager;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTextField;
import java.awt.Font;
import java.util.List;

/**
 * Cette classe est le panel reprsentant les films.
 *
 * @author Baptiste Wicht
 */
public final class MovieView extends PrincipalDataPanel implements CurrentObjectListener, ViewStateListener, IMovieView {
    private static final long serialVersionUID = -461914454879477388L;

    private JXTitledPanel panelMovie;

    private JTextField fieldTitle;
    private FileChooserPanel fieldFile;

    private NotesComboBoxModel modelNotes;
    private JComboBox comboNote;

    private ICategoriesView panelCategories;

    private JPanelMovieToolBar toolBar;
    private JXTree treeMovies;

    @Resource
    private IMovieController movieController;

    @Override
    @PostConstruct
    public void build() {
        PanelBuilder builder = new PanelBuilder(this);

        buildPanelListe(builder);
        buildPanelMovie(builder);
        
        addListeners();
    }
    
    private void addListeners() {
        treeMovies.addTreeSelectionListener(movieController);

        getModel().addViewStateListener(this);
        getModel().addDisplayListListener(this);

        getModel().addCurrentObjectListener(this);
    }

    /**
     * Build the internal panel film.
     * 
     * @param parent The parent builder. 
     */
    private void buildPanelMovie(PanelBuilder parent) {
        panelMovie = new JThequeTitledPanel("movie.panel.movie.title");
        panelMovie.setBorder(new DropShadowBorder());
        panelMovie.setTitleFont(panelMovie.getTitleFont().deriveFont(Font.BOLD));
        
        PanelBuilder builder = new PanelBuilder();
        
        builder.add(toolBar, builder.gbcSet(0, 0, GridBagUtils.HORIZONTAL, 2, 1));
        
        builder.addI18nLabel("movie.title", builder.gbcSet(0, 1));
        
        fieldTitle = builder.add(new JTextField(10), builder.gbcSet(1, 1));
        SwingUtils.addFieldLenghtLimit(fieldTitle, 100);
        fieldTitle.setEnabled(false);
        
        fieldFile = builder.add(new FileChooserPanel(), builder.gbcSet(0, 2, GridBagUtils.HORIZONTAL, 2, 1));
        fieldFile.setTextKey("movie.file");
        fieldFile.setEnabled(false);

        builder.addI18nLabel("movie.note", builder.gbcSet(0, 3));
        
        modelNotes = new NotesComboBoxModel();

        comboNote = builder.addComboBox(modelNotes, builder.gbcSet(1, 3));
        comboNote.setEnabled(false);
        comboNote.setRenderer(new NoteComboRenderer());

        builder.add(panelCategories.getImpl(), builder.gbcSet(0, 4, GridBagUtils.BOTH, 2, 1));

        panelMovie.setContentContainer(builder.getPanel());
        
        parent.add(panelMovie, parent.gbcSet(1, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 0.75, 1.0));
    }

    /**
     * Build the internal panel list.
     * 
     * @param parent The parent builder. 
     */
    private void buildPanelListe(PanelBuilder parent) {
        JXTitledPanel panelListe = new JThequeTitledPanel("movie.panel.list.title");
        panelListe.setBorder(new DropShadowBorder());
        panelListe.setTitleFont(panelListe.getTitleFont().deriveFont(Font.BOLD));

        PanelBuilder builder = new PanelBuilder();

        SortManager sorter = new SortManager();

        setTreeModel(sorter.createInitialModel(IMoviesService.DATA_TYPE));
        
        sort(IMoviesService.SORT_BY_CATEGORY);
        
        treeMovies = new JXTree(getTreeModel());
        treeMovies.setCellRenderer(new JThequeTreeCellRenderer());
        treeMovies.putClientProperty("JTree.lineStyle", "None");

        builder.addScrolled(treeMovies, builder.gbcSet(0, 0));

        panelListe.setContentContainer(builder.getPanel());
        
        parent.add(panelListe, parent.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 0.25, 1.0));
    }

    @Override
    public void objectChanged(ObjectChangedEvent event) {
        setCurrentFilm((MovieImpl) event.getObject());
    }

    /**
     * Cette mthode change le film actuellement affich.
     *
     * @param movie The new current film
     */
    private void setCurrentFilm(MovieImpl movie) {
        panelMovie.setTitle(movie.getAffichableText());

        fieldTitle.setText(movie.getTitle());
        fieldFile.setFilePath(movie.getFile());
        modelNotes.setSelectedItem(movie.getNote());

        panelCategories.reload(movie);
    }

    @Override
    public void stateChanged() {
        setEnabled(getModel().isEnabled());
    }

    @Override
    public void setEnabled(boolean enabled) {
        fieldTitle.setEnabled(enabled);
        fieldFile.setEnabled(enabled);
        panelCategories.setEnabled(enabled);
        comboNote.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return getModel().isEnabled();
    }

    @Override
    public MovieFormBean fillFilmFormBean() {
        MovieFormBean fb = new MovieFormBean();

        fb.setTitle(fieldTitle.getText());
        fb.setNote(modelNotes.getSelectedNote());
        fb.setFile(fieldFile.getFilePath());

        panelCategories.fillFilm(fb);

        return fb;
    }

    @Override
    public ToolbarView getToolbarView() {
        return toolBar;
    }

    @Override
    public String getDataType() {
        return IMoviesService.DATA_TYPE;
    }

    @Override
    protected JXTree getTree() {
        return treeMovies;
    }

    @Override
    public JComponent getImpl() {
        return this;
    }

    @Override
    public Integer getPosition() {
        return 1;
    }

    @Override
    public String getTitlekey() {
        return "data.titles.movie";
    }

    @Override
    public MoviesModel getModel() {
        return (MoviesModel) super.getModel();
    }

    @Override
    public void clear() {
    }

    @Override
    public ICategoriesView getPanelCategories() {
        return panelCategories;
    }

    @Override
    protected void validate(List<JThequeError> errors) {
        ValidationUtils.rejectIfEmpty(fieldTitle.getText(), "movie.title", errors);
        ValidationUtils.rejectIfLongerThan(fieldTitle.getText(), "movie.title", 150, errors);

        ValidationUtils.rejectIfEmpty(fieldFile.getFilePath(), "movie.file", errors);
        ValidationUtils.rejectIfLongerThan(fieldFile.getFilePath(), "movie.file", 150, errors);

        panelCategories.validate(errors);
    }

    @Override
    public JComponent getComponent() {
        return this;
    }

    @Override
    public Iterable<MovieImpl> getDisplayList() {
        return getModel().getDisplayList();
    }

    public void setToolBar(JPanelMovieToolBar toolBar) {
        this.toolBar = toolBar;
    }

    public void setPanelCategories(ICategoriesView panelCategories) {
        this.panelCategories = panelCategories;
    }
}