package org.jtheque.movies.views.impl.models;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.movies.persistence.od.MovieImpl;
import org.jtheque.movies.services.able.IMoviesService;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.listeners.ViewStateListener;
import org.jtheque.primary.view.impl.models.PrincipalDataModel;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;

import java.util.Collection;
import java.util.List;

/**
 * A model for the movies view.
 *
 * @author Baptiste Wicht
 */
public final class MoviesModel extends PrincipalDataModel {
    private boolean enabled;
    private MovieImpl currentMovie;

    private List<MovieImpl> displayList;

    @Resource
    private IMoviesService moviesService;

    /**
     * Init the model.
     */
    @PostConstruct
    private void init() {
        moviesService.addDataListener(this);
    }

    @Override
    @SuppressWarnings("unchecked")
    public void updateDisplayList(List<? extends Entity> datas) {
        getDisplayList().clear();

        Collection<MovieImpl> movies = (Collection<MovieImpl>) datas;

        if (movies == null) {
            getDisplayList().addAll(moviesService.getMovies());
        } else {
            getDisplayList().addAll(movies);
        }

        fireDisplayListChanged();
    }

    @Override
    public void updateDisplayList() {
        updateDisplayList(null);
    }

    /**
     * Return the display list.
     *
     * @return The display list
     */
    public List<MovieImpl> getDisplayList() {
        if (displayList == null) {
            displayList = moviesService.getMovies();
        }

        return displayList;
    }

    /**
     * Set if the view is enabled or not.
     *
     * @param enabled A boolean flag indicating if the view is enabled.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;

        fireStateChanged();
    }

    /**
     * Indicate if the view is enabled or not.
     *
     * @return true if the view is enabled else false.
     */
    public boolean isEnabled() {
        return enabled;
    }

    /**
     * Return the current movie.
     *
     * @return The current movie.
     */
    public MovieImpl getCurrentMovie() {
        return currentMovie;
    }

    /**
     * Set the current movie.
     *
     * @param currentMovie The new current movie
     */
    public void setCurrentMovie(MovieImpl currentMovie) {
        this.currentMovie = currentMovie;

        fireCurrentObjectChanged(new ObjectChangedEvent(this, currentMovie));
    }

    /**
     * Add a view state listener.
     *
     * @param listener The listener.
     */
    public void addViewStateListener(ViewStateListener listener) {
        getEventListenerList().add(ViewStateListener.class, listener);
    }

    /**
     * Remove a view state listener.
     *
     * @param listener The listener.
     */
    public void removeViewStateListener(ViewStateListener listener) {
        getEventListenerList().remove(ViewStateListener.class, listener);
    }

    /**
     * Fire a state changed event.
     */
    private void fireStateChanged() {
        ViewStateListener[] listeners = getEventListenerList().getListeners(ViewStateListener.class);

        for (ViewStateListener listener : listeners) {
            listener.stateChanged();
        }
    }
}